%% Target localization step
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
        old_refine_pos = inf(size(refine_pos));
        iter = 1;
        %translation search
        while iter <= params.refinement_iterations && any(old_refine_pos ~= refine_pos) 
            
            old_refine_sz = refine_sz;
            old_currentScaleFactor = currentScaleFactor;
            
            % Extract features at multiple resolutions
            sample_pos = round(refine_pos);
            det_sample_pos = sample_pos;
            sample_scale = old_currentScaleFactor*scaleFactors;
            xt = extract_features(img, sample_pos, sample_scale, features, global_fparams, feature_extract_info);
                        
            % Project sample
            xt_proj = project_sample(xt, projection_matrix);
            
            % Do windowing of features
            xt_proj = cellfun(@(feat_map, cos_window) bsxfun(@times, feat_map, cos_window), xt_proj, cos_window, 'uniformoutput', false);
            
            % Compute the fourier series
            xtf_proj = cellfun(@cfft2, xt_proj, 'uniformoutput', false);
            
            % Interpolate features to the continuous domain
            xtf_proj = interpolate_dft(xtf_proj, interp1_fs, interp2_fs);


            
            % Compute convolution for each feature block in the Fourier domain
            % and the sum over all blocks.
            scores_fs_feat{k1} = sum(bsxfun(@times, hf_full{k1}, xtf_proj{k1}), 3);
            scores_fs_sum = scores_fs_feat{k1};
            for k = block_inds
                scores_fs_feat{k} = sum(bsxfun(@times, hf_full{k}, xtf_proj{k}), 3);
                scores_fs_sum(1+pad_sz{k}(1):end-pad_sz{k}(1), 1+pad_sz{k}(2):end-pad_sz{k}(2),1,:) = ...
                    scores_fs_sum(1+pad_sz{k}(1):end-pad_sz{k}(1), 1+pad_sz{k}(2):end-pad_sz{k}(2),1,:) + ...
                    scores_fs_feat{k};
            end
            
            % Also sum over all feature blocks.
            % Gives the fourier coefficients of the convolution response.
            scores_fs = permute(gather(scores_fs_sum), [1 2 4 3]);
            
            % Optimize the continuous score function with Newton's method.
            [trans_row, trans_col, scale_ind] = optimize_scores(scores_fs, params.newton_iterations);
            
            % Compute the translation vector in pixel-coordinates and round
            % to the closest integer pixel.
            translation_vec = [trans_row, trans_col] .* (img_support_sz./output_sz) * old_currentScaleFactor * scaleFactors(scale_ind);
%             translation_vec = [trans_row, trans_col] .* (img_support_sz./output_sz) * old_currentScaleFactor;

%             scale_change_factor = scaleFactors(scale_ind);
            
            % update position
            old_refine_pos = refine_pos;
            pos = sample_pos + translation_vec;
            
            if params.clamp_position
                pos = max([1 1], min([size(img,1) size(img,2)], pos)); 
            end
            
            iter = iter + 1;
        end
        