%% Model update step
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract sample and init projection matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if seq.frame == 1
    % Extract image region for training sample
    sample_pos = round(refine_pos);
    sample_scale = currentScaleFactor;


    xl = extract_features(img, sample_pos, sample_scale, features, global_fparams, feature_extract_info); 

    % Do windowing of features
    xlw = cellfun(@(feat_map, cos_window) bsxfun(@times, feat_map, cos_window), xl, cos_window, 'uniformoutput', false);

    % Compute the fourier series
    xlf = cellfun(@cfft2, xlw, 'uniformoutput', false);

    % Interpolate features to the continuous domain
    xlf = interpolate_dft(xlf, interp1_fs, interp2_fs);

    % New sample to be added
    xlf = compact_fourier_coeff(xlf);

    % Shift sample
    shift_samp = 2*pi * (refine_pos - sample_pos) ./ (sample_scale * img_support_sz);
    xlf = shift_sample(xlf, shift_samp, kx, ky);

    % Init the projection matrix
    projection_matrix = init_projection_matrix(xl, sample_dim, params);

    % Project sample
    xlf_proj = project_sample(xlf, projection_matrix);

    clear xlw
elseif params.learning_rate > 0
    if ~params.use_detection_sample % params.use_detection_sample = true 
        % Extract image region for training sample
        sample_pos = round(refine_pos); %%%%%%%%%%%%%%%%%%%%%%
        sample_scale = currentScaleFactor;
        xl = extract_features(img, sample_pos, currentScaleFactor, features, global_fparams, feature_extract_info);

        % Project sample
        xl_proj = project_sample(xl, projection_matrix);

        % Do windowing of features
        xl_proj = cellfun(@(feat_map, cos_window) bsxfun(@times, feat_map, cos_window), xl_proj, cos_window, 'uniformoutput', false);

        % Compute the fourier series
        xlf1_proj = cellfun(@cfft2, xl_proj, 'uniformoutput', false);

        % Interpolate features to the continuous domain
        xlf1_proj = interpolate_dft(xlf1_proj, interp1_fs, interp2_fs);

        % New sample to be added
        xlf_proj = compact_fourier_coeff(xlf1_proj);
    else
        if params.debug
            % Only for visualization
            xl = cellfun(@(xt) xt(:,:,:,scale_ind), xt, 'uniformoutput', false);
        end

        % Use the sample that was used for detection
        sample_scale = sample_scale(scale_ind);
        xlf_proj = cellfun(@(xf) xf(:,1:(size(xf,2)+1)/2,:,scale_ind), xtf_proj, 'uniformoutput', false);
    end

    % Shift the sample so that the target is centered
    shift_samp = 2*pi * (refine_pos - sample_pos) ./ (sample_scale * img_support_sz);
    xlf_proj = shift_sample(xlf_proj, shift_samp, kx, ky);
end

% The permuted sample is only needed for the CPU implementation
if ~params.use_gpu
    xlf_proj_perm = cellfun(@(xf) permute(xf, [4 3 1 2]), xlf_proj, 'uniformoutput', false);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GMM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if params.use_sample_merge  
    % Update the samplesf to include the new sample. The distance
    % matrix, kernel matrix and prior weight are also updated
    if params.use_gpu
        [merged_sample, new_sample, merged_sample_id, new_sample_id, distance_matrix, gram_matrix, prior_weights] = ...
            update_sample_space_model_gpu(samplesf, xlf_proj, distance_matrix, gram_matrix, prior_weights,...
            num_training_samples,params);
    else
        [merged_sample, new_sample, merged_sample_id, new_sample_id, distance_matrix, gram_matrix, prior_weights] = ...
            update_sample_space_model(samplesf, xlf_proj_perm, distance_matrix, gram_matrix, prior_weights,...
            num_training_samples,params);
    end

    if num_training_samples < params.nSamples
        num_training_samples = num_training_samples + 1;
    end
else
    % Do the traditional adding of a training sample and weight update
    % of C-COT
    [prior_weights, replace_ind] = update_prior_weights(prior_weights, gather(sample_weights), latest_ind, seq.frame, params);
    latest_ind = replace_ind;

    merged_sample_id = 0;
    new_sample_id = replace_ind;
    if params.use_gpu
        new_sample = xlf_proj;
    else
        new_sample = xlf_proj_perm;
    end
end

if seq.frame > 1 && params.learning_rate > 0 || seq.frame == 1 && ~params.update_projection_matrix
    % Insert the new training sample
    for k = 1:num_feature_blocks
        if params.use_gpu
            if merged_sample_id > 0
                samplesf{k}(:,:,:,merged_sample_id) = merged_sample{k};
            end
            if new_sample_id > 0
                samplesf{k}(:,:,:,new_sample_id) = new_sample{k};
            end
        else
            if merged_sample_id > 0
                samplesf{k}(merged_sample_id,:,:,:) = merged_sample{k};
            end
            if new_sample_id > 0
                samplesf{k}(new_sample_id,:,:,:) = new_sample{k};
            end
        end
    end
end

sample_weights = cast(prior_weights, 'like', params.data_type);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Train model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
train_tracker = (seq.frame < params.skip_after_frame) || (frames_since_last_train >= params.train_gap); % ��ʼ��֡��ÿ��֡��Ҫѵ��
if train_tracker     
    % Used for preconditioning
    new_sample_energy = cellfun(@(xlf) abs(xlf .* conj(xlf)), xlf_proj, 'uniformoutput', false);

    if seq.frame == 1
        % Initialize stuff for the filter learning

        % Initialize Conjugate Gradient parameters
        sample_energy = new_sample_energy;
        CG_state = [];

        if params.update_projection_matrix
            % Number of CG iterations per GN iteration 
            init_CG_opts.maxit = ceil(params.init_CG_iter / params.init_GN_iter);

            hf = cell(2,1,num_feature_blocks);
            proj_energy = cellfun(@(P, yf) 2*sum(abs(yf(:)).^2) / sum(feature_dim) * ones(size(P), 'like', params.data_type), projection_matrix, yf, 'uniformoutput', false);
        else
            CG_opts.maxit = params.init_CG_iter; % Number of initial iterations if projection matrix is not updated

            hf = cell(1,1,num_feature_blocks);
        end

        % Initialize the filter with zeros
        for k = 1:num_feature_blocks
            hf{1,1,k} = zeros([filter_sz(k,1) (filter_sz(k,2)+1)/2 sample_dim(k)], 'like', params.data_type_complex);
        end
    else
        CG_opts.maxit = params.CG_iter;

        % Update the approximate average sample energy using the learning
        % rate. This is only used to construct the preconditioner.
        sample_energy = cellfun(@(se, nse) (1 - params.learning_rate) * se + params.learning_rate * nse, sample_energy, new_sample_energy, 'uniformoutput', false);
    end

    % Do training   
    if seq.frame == 1 && params.update_projection_matrix
        if params.debug
            projection_matrix_init = projection_matrix;
        end

        % Initial Gauss-Newton optimization of the filter and
        % projection matrix.
        if params.use_gpu
            [hf, projection_matrix, res_norms] = train_joint_gpu(hf, projection_matrix, xlf, yf, reg_filter, sample_energy, reg_energy, proj_energy, params, init_CG_opts);
        else
            [hf, projection_matrix, res_norms] = train_joint(hf, projection_matrix, xlf, yf, reg_filter, sample_energy, reg_energy, proj_energy, params, init_CG_opts);
        end

        % Re-project and insert training sample
        xlf_proj = project_sample(xlf, projection_matrix);
        for k = 1:num_feature_blocks
            if params.use_gpu
                samplesf{k}(:,:,:,1) = xlf_proj{k};
            else
                samplesf{k}(1,:,:,:) = permute(xlf_proj{k}, [4 3 1 2]);
            end
        end

        % Update the gram matrix since the sample has changed
        if strcmp(params.distance_matrix_update_type, 'exact')
            % Find the norm of the reprojected sample
            new_train_sample_norm =  0;

            for k = 1:num_feature_blocks
                new_train_sample_norm = new_train_sample_norm + real(gather(2*(xlf_proj{k}(:)' * xlf_proj{k}(:))));% - reshape(xlf_proj{k}(:,end,:,:), [], 1, 1)' * reshape(xlf_proj{k}(:,end,:,:), [], 1, 1));
            end

            gram_matrix(1,1) = new_train_sample_norm;
        end

        if params.debug
            norm_proj_mat_init = sqrt(sum(cellfun(@(P) gather(norm(P(:))^2), projection_matrix_init)));
            norm_proj_mat = sqrt(sum(cellfun(@(P) gather(norm(P(:))^2), projection_matrix)));
            norm_proj_mat_change = sqrt(sum(cellfun(@(P,P2) gather(norm(P(:) - P2(:))^2), projection_matrix_init, projection_matrix)));
            fprintf('Norm init: %f, Norm final: %f, Matrix change: %f\n', norm_proj_mat_init, norm_proj_mat, norm_proj_mat_change / norm_proj_mat_init);
        end
    else  
        % Do Conjugate gradient optimization of the filter  
        if params.use_gpu
            [hf, res_norms, CG_state] = train_filter_gpu(hf, samplesf, yf, reg_filter, sample_weights, sample_energy, reg_energy, params, CG_opts, CG_state);
        else
            [hf, res_norms, CG_state] = train_filter(hf, samplesf, yf, reg_filter, sample_weights, sample_energy, reg_energy, params, CG_opts, CG_state);
        end
    end

    % Reconstruct the full Fourier series
    hf_full = full_fourier_coeff(hf);

    frames_since_last_train = 0;
else
    frames_since_last_train = frames_since_last_train+1;
end


